#! /bin/sh -f
set -e

REPEATMASKER=RepeatMasker
TEST=test
TEMP_DIR=temp.d
option1="one-to-one"
option2="one-to-many"
option3="many-to-many"
option4="inf-dup"
code=../bin
EXON_DIR=exons.d

if [ $# != 1 ] && [ $# != 2 ]
then
        echo "Usage: conversion.sh Species_tree-file [--no_rm]"
        exit 1
fi

ls seq.d > all_species.txt
$code/check_species_tree $1 all_species.txt

mkdir -p rm_out.d
mkdir -p new_seq.d

if [ $# == 1 ]
then
	rm -f new_seq.d/*
	for sp_name in `ls seq.d`
	do
        	cp seq.d/$sp_name new_seq.d/$sp_name
	done

	for sp_name in `ls new_seq.d`
	do
        	echo ">"$sp_name > name
        	sed -e '1d' new_seq.d/$sp_name > new_seq.d/$sp_name.1
        	cat name new_seq.d/$sp_name.1 > new_seq.d/$sp_name
        	rm new_seq.d/$sp_name.1
	done
	rm name

	for sp_name in `ls new_seq.d`
	do
		$TEST -d $TEMP_DIR || mkdir $TEMP_DIR
		cp new_seq.d/$sp_name $TEMP_DIR/$sp_name
		$REPEATMASKER -xsmall $TEMP_DIR/$sp_name
		$code/check_mask seq.d/$sp_name $TEMP_DIR/$sp_name.masked > new_seq.d/$sp_name
		#cp $TEMP_DIR/$sp_name.masked new_seq.d/$sp_name
		cp $TEMP_DIR/$sp_name.out rm_out.d/$sp_name.out
		rm -rf $TEMP_DIR
	done
elif [ $2 == "--no_rm" ]
then
	for sp_name in `ls seq.d`
	do
		if [ ! -f new_seq.d/$sp_name ] || [ ! -f rm_out.d/$sp_name.out ]
		then
			echo "Sequence $sp_name is not masked. Please remove --no_rm option."
			exit 1
		fi
	done
else
	echo "Usage: conversion.sh Species_tree-file [--no_rm]"
        exit 1
fi

mkdir -p ortho.d
mkdir -p ortho.d/self.d
mkdir -p ortho.d/pair.d
mkdir -p ortho.d/ch_pair.d
mkdir -p ortho.d/ch_self.d
mkdir -p ortho.d/data.d
mkdir -p ortho.d/$option1-ortho.d
mkdir -p ortho.d/$option2-ortho.d
mkdir -p ortho.d/$option3-ortho.d

for sp_name in `ls new_seq.d`
do
	$code/lastz T=2 Y=3400 new_seq.d/$sp_name --self --nomirror --format=maf > ortho.d/self.d/$sp_name.maf
done

for sp_name1 in `ls new_seq.d`
do
	for sp_name2 in `ls new_seq.d`
	do
		if [ $sp_name1 != $sp_name2 ]
		then
			$code/lastz T=2 Y=3400 new_seq.d/$sp_name1 new_seq.d/$sp_name2 --format=maf > ortho.d/pair.d/$sp_name1.$sp_name2.maf
			$code/cage_mask ortho.d/self.d/$sp_name1.maf ortho.d/pair.d/$sp_name1.$sp_name2.maf a.maf ab_new.maf 1
			$code/cage_mask ortho.d/self.d/$sp_name2.maf ab_new.maf b.maf new.maf 2
			if [ -d "$EXON_DIR" ]; then
				$code/cage_chain a.maf new_a.maf rm_out.d/$sp_name1.out rm_out.d/$sp_name1.out exons.d/$sp_name1.exons exons.d/$sp_name1.exons
                        	$code/cage_chain b.maf new_b.maf rm_out.d/$sp_name2.out rm_out.d/$sp_name2.out exons.d/$sp_name2.exons exons.d/$sp_name2.exons
                        	$code/cage_chain new.maf new_ab.maf rm_out.d/$sp_name1.out rm_out.d/$sp_name2.out exons.d/$sp_name1.exons exons.d/$sp_name2.exons
			else
				$code/cage_chain a.maf new_a.maf rm_out.d/$sp_name1.out rm_out.d/$sp_name1.out
				$code/cage_chain b.maf new_b.maf rm_out.d/$sp_name2.out rm_out.d/$sp_name2.out
				$code/cage_chain new.maf new_ab.maf rm_out.d/$sp_name1.out rm_out.d/$sp_name2.out
			fi
			cp new_a.maf ortho.d/ch_self.d/$sp_name1.maf
			cp new_b.maf ortho.d/ch_self.d/$sp_name2.maf
			cp new_ab.maf ortho.d/ch_pair.d/$sp_name1.$sp_name2.maf
			cat new_a.maf > ortho.d/data.d/$sp_name1.$sp_name2.maf
			cat new_b.maf >> ortho.d/data.d/$sp_name1.$sp_name2.maf
			cat new_ab.maf >> ortho.d/data.d/$sp_name1.$sp_name2.maf
			echo "orthologous mappings between $sp_name1 and $sp_name2"
			$code/cage_ortho ortho.d/data.d/$sp_name1.$sp_name2.maf $option1 > ortho.d/$option1-ortho.d/$sp_name1.$sp_name2.maf 
			$code/cage_ortho ortho.d/data.d/$sp_name1.$sp_name2.maf $option2 > ortho.d/$option2-ortho.d/$sp_name1.$sp_name2.maf 
			$code/cage_ortho ortho.d/data.d/$sp_name1.$sp_name2.maf $option3 > ortho.d/$option3-ortho.d/$sp_name1.$sp_name2.maf 
		fi
	done
done

rm -f a.maf b.maf ab_new.maf new.maf new_a.maf new_b.maf new_ab.maf

# chech if all orthologous files are not empty
for file in `ls ortho.d/one-to-many-ortho.d/`
do
        if [ ! -s ortho.d/one-to-many-ortho.d/$file ]
        then
                echo "ortho.d/one-to-many-ortho.d/$file is empty"
                exit 1
        fi
done

for file in `ls ortho.d/many-to-many-ortho.d/`
do
        if [ ! -s ortho.d/many-to-many-ortho.d/$file ]
        then
                echo "ortho.d/many-to-many-ortho.d/$file is empty"
                exit 1
        fi
done

# get conversion
for sp_name1 in `ls new_seq.d`
do
	mkdir -p $sp_name1.d
	echo $sp_name1
	$code/lastz new_seq.d/$sp_name1 --self Q=../human_dog.q --maf > $sp_name1.d/$sp_name1.maf
        $code/remove_repeats $sp_name1.d/$sp_name1.maf > $sp_name1.d/$sp_name1.remove_repeats.maf
	rm $sp_name1.d/$sp_name1.maf
        $code/remove_lower_tri $sp_name1.d/$sp_name1.remove_repeats.maf > $sp_name1.d/$sp_name1.remove_lower_tri.maf
#       rm $sp_name1.d/$sp_name1.remove_repeats.maf    # keep this for Gmaj
        $code/chain $sp_name1.d/$sp_name1.remove_lower_tri.maf > $sp_name1.d/$sp_name1.chain
        rm $sp_name1.d/$sp_name1.remove_lower_tri.maf
	for sp_name2 in `ls new_seq.d`
	do
		if [ $sp_name1 != $sp_name2 ]
		then
			mkdir -p $sp_name1.d/$sp_name2.d
			$code/md_quadruplet_entire_region $sp_name1.d/$sp_name1.chain ortho.d/$option2-ortho.d/$sp_name1.$sp_name2.maf ortho.d/$option3-ortho.d/$sp_name1.$sp_name2.maf > $sp_name1.d/$sp_name2.d/all.md
		fi
	done
done

# calculate p-values

echo "get maximum descent"
rm -f all.md
for sp_name1 in `ls new_seq.d`
do
	for sp_name2 in `ls new_seq.d`
	do
		if [ $sp_name1 != $sp_name2 ] 
		then
			echo "#$sp_name1.$sp_name2" >> all.md
			cat $sp_name1.d/$sp_name2.d/all.md >> all.md
		fi
	done
done

echo "calculate p-value and find conversions"
$code/3seq_2D_cluster all.md > all.pvalue
$code/separate_pvalue all.pvalue
rm -f all.gc
echo
echo "======================Results of Conversion=========================="
echo "Species                 Paralogous pairs        Gene conversion"
for sp_name1 in `ls new_seq.d`
do
	parameters=""
	for sp_name2 in `ls new_seq.d`
	do
		if [ $sp_name1 != $sp_name2 ]
		then
			parameters=`echo "$parameters $sp_name1.d/$sp_name2.d/all.pvalue"`
		fi
	done
	$code/FDR_cluster $parameters > FDR.txt
	pvalue=`$code/FDR_test FDR.txt`
	$code/combine_all_pvalue_cluster $parameters $pvalue $sp_name1.d/all.corrected.pvalue
	$code/show_GC_cluster $sp_name1.d/all.corrected.pvalue >> all.gc
done
	
$code/GC_history $1 all.gc > all.remove_redundancy.gc
cat species_tree_with_index.txt all_1.gc > all.gc
rm -f species_tree_with_index.txt
rm -f all_1.gc

rm -rf ortho.d/self.d
rm -rf ortho.d/pair.d
rm -rf ortho.d/ch_self.d
rm -rf ortho.d/data.d
rm -rf ortho.d/$option1-ortho.d

echo
echo 'To view the results with Gmaj (requires Java), run a command like this:'
echo '  ../gmaj.sh refseq_name [genomic_offset] ["title"]'
echo 'Examples:'
echo '  ../gmaj.sh human'
echo '  ../gmaj.sh human 31334805'
echo '  ../gmaj.sh human 31334805 "Gene conversions in the human CCL region"'
echo '(If you want to specify a title but no offset, use 0 for the offset.)'
