/* md_quadruplet_best - calculate m, n, and k for all tests. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#define MAX_GAP 5000	
#define MIN_COVERAGE 0.5
#define MIN_MATCH 0.8	
#define MAX_GC_RATIO 0.9

typedef struct Net {
	int start, end, start2, end2, chain_id;
	char *net1, *net2, chr2[100], orient;
} net_t;

typedef struct Net_level {
	int chain_id, end, chr_size;
	char chr2[100];
} net_level_t;

typedef struct Aln {
	int start1, len1, src_len1, start2, len2, src_len2;
	char text1[1000000], text2[1000000];
	struct Aln *next;
} aln_t;

typedef struct Chain {
	char chr1[100], chr2[100], orient;
	int start1, end1, start2, end2;
	aln_t *aln;
} chain_t;

net_level_t net_level[1000];
int net_level_num;
net_t net[2][1000000];
int rescored[2][1000000];
int net_num[2];
char buf[1000000];
char seqs[2][3][1000000];
int seq2_pos[2][1000000];
int d_beg_index[10000], d_end_index[10000], d_index_num;
int sites_num[2];
int IS[3][1000000], IS_num[3], IS_sites[4][1000000], IS_ref[1000000];
int IS_quadruplet[1000000], IS_quadruplet_num, IS_quadruplet_sites[2][1000000];
int m[2], n[2], max_descent[3], beg_index = 0, end_index = 0;
double identity;
int max_len, min_len, max_beg1, min_beg1, min_end1, max_end1, max_beg2, min_beg2, min_end2, max_end2, start[2], end[2];
int offset[3];
int site_beg[2], site_end[2];
int mismatch_net[100000];
int mismatch_net_num;
int long_seqs;
//score parameters
int fill_score = -100, T = 1, Z = 1, O = 400, E = 30, X = 910, Y = 9400, K = 3000, L = 3000; 
int score[4][4] = {{91, -114, -31, -123}, {-114, 100, -125, -31}, {-31, -125, 100, -114}, {-123, -31, -114, 91}};

/* Read score file. */
void read_score(char *filename) {
	FILE *fp;
	char buf[10000], *temp_str;
	
	fp = fopen(filename, "r");
	while(fgets(buf, 10000, fp)) {
		if(buf[0] == '#')
			continue;
		if(strncmp(buf, "fill_score", 10) == 0) {
			strtok(buf, "=");
			temp_str = strtok(NULL, "=");
			sscanf(temp_str, "%d", &fill_score);
		}
		else if (buf[1] == '=') {
			if(buf[0] == 'T') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &T);
			}
			else if(buf[0] == 'O') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &O);
			}
			else if(buf[0] == 'E') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &E);
			}
			else if(buf[0] == 'X') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &X);
			}
			else if(buf[0] == 'Y') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &Y);
			}
			else if(buf[0] == 'K') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &K);
			}
			else if(buf[0] == 'L') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &L);
			}
		}
		else if(buf[0] == 'A')
			sscanf(buf, "A %d %d %d %d", &score[0][0], &score[0][1], &score[0][2], &score[0][3]);
		else if(buf[0] == 'C')
			sscanf(buf, "C %d %d %d %d", &score[1][0], &score[1][1], &score[1][2], &score[1][3]);
		else if(buf[0] == 'G')
			sscanf(buf, "G %d %d %d %d", &score[2][0], &score[2][1], &score[2][2], &score[2][3]);
		else if(buf[0] == 'T')
			sscanf(buf, "T %d %d %d %d", &score[3][0], &score[3][1], &score[3][2], &score[3][3]);
	}
}

int cal_score(char A, char B, int gap_opend) {
	if(A == '-' || B == '-') {
		if(gap_opend == 1)
			return -E;
		else
			return -O;
	}
	if(toupper(A) == 'A') {
		if(toupper(B) == 'A')
			return score[0][0];
		else if(toupper(B) == 'C')
			return score[0][1];
		else if(toupper(B) == 'G')
			return score[0][2];
		else if(toupper(B) == 'T')
			return score[0][3];
		else
			return fill_score;
	}
	else if(toupper(A) == 'C') {
		if(toupper(B) == 'A')
			return score[1][0];
		else if(toupper(B) == 'C')
			return score[1][1];
		else if(toupper(B) == 'G')
			return score[1][2];
		else if(toupper(B) == 'T')
			return score[1][3];
		else
			return fill_score;
	}
	else if(toupper(A) == 'G') {
		if(toupper(B) == 'A')
			return score[2][0];
		else if(toupper(B) == 'C')
			return score[2][1];
		else if(toupper(B) == 'G')
			return score[2][2];
		else if(toupper(B) == 'T')
			return score[2][3];
		else
			return fill_score;
	}
	else if(toupper(A) == 'T') {
		if(toupper(B) == 'A')
			return score[3][0];
		else if(toupper(B) == 'C')
			return score[3][1];
		else if(toupper(B) == 'G')
			return score[3][2];
		else if(toupper(B) == 'T')
			return score[3][3];
		else
			return fill_score;
	}
	else
		return fill_score;
}

/* Recalculate the scores for all alignment based on the score file.  */
void rescoring(int ref, int i) {
	int j, k, max_score, cur_score, gap_opend, aln_beg, aln_max, gap_free, net_len;

	max_score = cur_score = gap_opend = aln_beg = aln_max = 0;
	gap_free = 1;
	net_len = strlen(net[ref-1][i].net1);
	for(j=0; j<net_len; j++) {
		cur_score += cal_score(net[ref-1][i].net1[j], net[ref-1][i].net2[j], gap_opend);	
		if(net[ref-1][i].net1[j] == '-' || net[ref-1][i].net2[j] == '-') {
			gap_opend = 1;
			gap_free = 0;
		}
		else
			gap_opend = 0;
				
		if(cur_score >= max_score) {
			max_score = cur_score;
			aln_max = j;
		}

		if(cur_score < 0 || (max_score - cur_score) > Y || j == net_len - 1) {
			if(max_score < L) //throw away whole segment
				for(k=aln_beg; k<=j; k++) {
					if(net[ref-1][i].net1[k] != '-')
						net[ref-1][i].net1[k] = 'X';
					if(net[ref-1][i].net2[k] != '-')
						net[ref-1][i].net2[k] = 'X';
				}
			else {	//throw away the dropping region
				for(k=aln_max+1; k<=j; k++) {
					if(net[ref-1][i].net1[k] != '-')
						net[ref-1][i].net1[k] = 'X';
					if(net[ref-1][i].net2[k] != '-')
						net[ref-1][i].net2[k] = 'X';
				}
			}
			aln_beg = aln_max = j + 1;
			max_score = cur_score = 0;
			gap_free = 1;
		}
	}
	
	return;
}

/* Read .net and .axt files. */
int read_net(char *net_fn, char *axt_fn, int ref) {
	FILE *net_fp, *axt_fp;
	char buf[1000000], chr1[100], chr2[100], orient, net_class[10], net_chr2[100], net_orient, id[100];
	int i, index, start1, end1, start2, end2, net_start1, net_size1, net_start2, net_size2, chain_id;
	
	net_fp = fopen(net_fn, "r");
	axt_fp = fopen(axt_fn, "r");
	net_num[ref-1] = 0;
	net_level_num = 0;
	fgets(buf, 1000000, net_fp);
	while(buf[0] == '#')
		fgets(buf, 1000000, net_fp);
	while(fgets(buf, 1000000, axt_fp)) {
		while(buf[0] == '#')
			fgets(buf, 1000000, axt_fp);
		sscanf(buf, "%d %s %d %d %s %d %d %c", &index, chr1, &start1, &end1, chr2, &start2, &end2, &orient);
		// test length
		if((end1 - start1 + 1) > 1000000 || (end2 - start2 + 1) > 1000000) {
			printf("net is too long : %d - %d\n", end1 - start1 + 1, end2 - start2 + 1);
			exit(1);
		}

		net[ref-1][net_num[ref-1]].start = start1;
		net[ref-1][net_num[ref-1]].end = end1;
		strcpy(net[ref-1][net_num[ref-1]].chr2, chr2);
		net[ref-1][net_num[ref-1]].orient = orient;
		fgets(buf, 1000000, axt_fp);
		net[ref-1][net_num[ref-1]].net1 = (char *)malloc(sizeof(char) * (strlen(buf) + 1));
		strcpy(net[ref-1][net_num[ref-1]].net1, buf);
		fgets(buf, 1000000, axt_fp);
		net[ref-1][net_num[ref-1]].net2 = (char *)malloc(sizeof(char) * (strlen(buf) + 1));
		strcpy(net[ref-1][net_num[ref-1]].net2, buf);
		
		//read net_file
		if(fgets(buf, 1000000, net_fp) != NULL) {
			sscanf(buf, "%s %d %d %s %c %d %d %s %d", net_class, &net_start1, &net_size1, net_chr2, &net_orient, &net_start2, &net_size2, id, &chain_id);
			while(strcmp(net_class, "fill") != 0) {
				if(fgets(buf, 1000000, net_fp) == NULL) 
					break;
				sscanf(buf, "%s %d %d %s %c %d %d %s %d", net_class, &net_start1, &net_size1, net_chr2, &net_orient, &net_start2, &net_size2, id, &chain_id);
			}
		}
			
		while(start1 != net_start1 + 1 || strcmp(chr2, net_chr2) != 0 || orient != net_orient || strcmp(net_class, "fill") != 0) { // nets which is not fill
			if(net_level_num == 0) {
				printf("wrong net-file format!\n");
				exit(1);
			}
			if(strcmp(chr2, net_level[net_level_num - 1].chr2) != 0) {
				printf("chromosomes are not matched %s %s!\n", chr2, net_level[net_level_num - 1].chr2);
				exit(1);
			}

			if(orient == '+') {
				net[ref-1][net_num[ref-1]].start2 = start2;
				net[ref-1][net_num[ref-1]].end2 = end2;
			}
			else {
				net[ref-1][net_num[ref-1]].start2 = net_level[net_level_num - 1].chr_size - end2;
				net[ref-1][net_num[ref-1]].end2 = net_level[net_level_num - 1].chr_size - start2;
			}
			net[ref-1][net_num[ref-1]].chain_id = net_level[net_level_num - 1].chain_id;

			if(net_level[net_level_num - 1].end == end1)
				net_level_num--;

			fgets(buf, 1000000, axt_fp);
			net_num[ref-1]++;

			if(!fgets(buf, 1000000, axt_fp)) {
				fclose(net_fp);
				fclose(axt_fp);
				return 0;
			}
			
			sscanf(buf, "%d %s %d %d %s %d %d %c", &index, chr1, &start1, &end1, chr2, &start2, &end2, &orient);
			// test length
			if((end1 - start1 + 1) > 1000000 || (end2 - start2 + 1) > 1000000) {
				printf("net is too long : %d - %d\n", end1 - start1 + 1, end2 - start2 + 1);
				exit(1);
			}

			net[ref-1][net_num[ref-1]].start = start1;
			net[ref-1][net_num[ref-1]].end = end1;
			strcpy(net[ref-1][net_num[ref-1]].chr2, chr2);
			net[ref-1][net_num[ref-1]].orient = orient;
			fgets(buf, 1000000, axt_fp);
			net[ref-1][net_num[ref-1]].net1 = (char *)malloc(sizeof(char) * (strlen(buf) + 1));
			strcpy(net[ref-1][net_num[ref-1]].net1, buf);
			fgets(buf, 1000000, axt_fp);
			net[ref-1][net_num[ref-1]].net2 = (char *)malloc(sizeof(char) * (strlen(buf) + 1));
			strcpy(net[ref-1][net_num[ref-1]].net2, buf);
		}
		if(strcmp(id, "id") != 0) {
			printf("wrong chain id!\n");
			exit(1);
		}
		net[ref-1][net_num[ref-1]].chain_id = chain_id;
		// check if the fill contains more than one nets
		if(end1 == net_start1 + net_size1) { //only contains one net
			net[ref-1][net_num[ref-1]].start2 = net_start2 + 1;
			net[ref-1][net_num[ref-1]].end2 = net_start2 + net_size2;
		}
		else { // more than one nets			
			net_level[net_level_num].chain_id = chain_id;
			net_level[net_level_num].end = net_start1 + net_size1;
			strcpy(net_level[net_level_num].chr2, chr2);
			if(orient == '-')
				net_level[net_level_num].chr_size = start2 + net_start2 + net_size2;
			else
				net_level[net_level_num].chr_size = 0;
			net_level_num++;

			if(orient == '+') {
				net[ref-1][net_num[ref-1]].start2 = start2;
				net[ref-1][net_num[ref-1]].end2 = end2;
			}
			else {
				net[ref-1][net_num[ref-1]].start2 = net_level[net_level_num - 1].chr_size - end2;
				net[ref-1][net_num[ref-1]].end2 = net_level[net_level_num - 1].chr_size - start2;
			}
		}

		fgets(buf, 1000000, axt_fp);
		net_num[ref-1]++;
	}
	fclose(net_fp);
	fclose(axt_fp);
	
	for(i=0; i<net_num[ref-1]; i++)
		rescored[ref-1][i] = 0;

	return 0;
}

/* Get the informative sites and calculate m and n. */
void find_IS(int ref) {
	int i;
	
	IS_num[ref-1] = 0;
	m[ref-1] = 0;
	n[ref-1] = 0;
	
	for(i=site_beg[ref-1]; i<site_end[ref-1]; i++) {
		//there are gaps in a site
		if((seqs[ref-1][0][i] == '-') || (seqs[ref-1][1][i] == '-') || (seqs[ref-1][2][i] == '-') || (seqs[ref-1][0][i] == 'N') || (seqs[ref-1][1][i] == 'N') || (seqs[ref-1][2][i] == 'N') || (seqs[ref-1][0][i] == 'n') || (seqs[ref-1][1][i] == 'n') || (seqs[ref-1][2][i] == 'n'))
			continue;
		//all are the same in a site
		if((toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][1][i])) && (toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][2][i])))
			continue;
		// all are different in a site
		else if((toupper(seqs[ref-1][0][i]) != toupper(seqs[ref-1][1][i])) && (toupper(seqs[ref-1][0][i]) != toupper(seqs[ref-1][2][i])) && (toupper(seqs[ref-1][1][i]) != toupper(seqs[ref-1][2][i])))
			continue;
		//type P
		else if(toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][2][i])) {
			IS[ref-1][IS_num[ref-1]] = 1;
			m[ref-1]++;
		}
		//type Q	
		else if(toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][1][i])) {
			IS[ref-1][IS_num[ref-1]] = 2;
			n[ref-1]++;
		}
		else
			continue;
		
		IS_sites[ref-1][IS_num[ref-1]] = i;
		IS_num[ref-1]++;
	}
}

/* Merge the informative sites for two triplet tests. */
void combine_IS(char orient) {
	int IS_index[2], i;

	if(orient == '+') {
		IS_index[0] = IS_index[1] = 0;
		IS_num[2] = 0;
		while(IS_index[0] < IS_num[0] || IS_index[1] < IS_num[1]) {
			if(IS_index[0] >= IS_num[0]) {
				for(i=IS_index[1]; i<IS_num[1]; i++) {
					IS[2][IS_num[2]] = IS[1][i];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][i]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][i];
					IS_num[2]++;
				}
				break;
			}
			else if(IS_index[1] >= IS_num[1]) {
				for(i=IS_index[0]; i<IS_num[0]; i++) {
					IS[2][IS_num[2]] = IS[0][i];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][i];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][i]];
					IS_num[2]++;
				}
				break;
			}
			else {
				if((start[0] + IS_sites[0][IS_index[0]]) < seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_index[0]++;
				}
				else if ((start[0] + IS_sites[0][IS_index[0]]) > seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[1]++;
				}
				else {	
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_num[2]++;
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[0]++;
					IS_index[1]++;
				}
				IS_num[2]++;
			}   
		}
	} 
	else {
		IS_index[0] = 0;
		IS_index[1] = IS_num[1] - 1;
		IS_num[2] = 0;
		while(IS_index[0] < IS_num[0] || IS_index[1] >= 0) {
			//printf("%d\t%d\n", IS_index[0], IS_index[1]);
			if(IS_index[0] >= IS_num[0]) {
				for(i=IS_index[1]; i>=0; i--) {
					IS[2][IS_num[2]] = IS[1][i];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][i]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][i];
					IS_num[2]++;
				}
				break;
			}
			else if(IS_index[1] < 0) {
				for(i=IS_index[0]; i<IS_num[0]; i++) {
					IS[2][IS_num[2]] = IS[0][i];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][i];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][i]];
					IS_num[2]++;
				}
				break;
			}
			else {
				if((start[0] + IS_sites[0][IS_index[0]]) < seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_index[0]++;
				}
				else if ((start[0] + IS_sites[0][IS_index[0]]) > seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[1]--;
				}
				else {
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_num[2]++;
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[0]++;
					IS_index[1]--;
				}
				IS_num[2]++;
			}   
		}
	}
}


/* Calculate the maximum decent for each test. */
void find_max_descent(int ref) {
	int max_height = 0, height = 0, descent = 0, i, max_height_index = 0;
	
	max_descent[ref-1] = 0;
	beg_index = end_index = 0;
	for(i=0; i<IS_num[ref-1]; i++) {
		if(IS[ref-1][i] == 1) {
			height++;
			if( height > max_height ) { 
				max_height = height;
				max_height_index = i;
			}
			descent = max_height - height;
			if( descent >= max_descent[ref-1] ) { 
				max_descent[ref-1] = descent;
				beg_index = max_height_index;
				end_index = i;
			}
		}
		else if(IS[ref-1][i] == 2) {
			height--;
			if( height > max_height ) { 
				max_height = height;
				max_height_index = i;
			}
			descent = max_height - height;
			if( descent >= max_descent[ref-1] ) {
				max_descent[ref-1] = descent;
				beg_index = max_height_index;
				end_index = i;
			}
		}
	}
	
	if(long_seqs == 1 || ref < 3)
		return;
	
	min_len = max_len = 0;
	min_len = IS_sites[2][end_index] - IS_sites[2][beg_index+1] + 1;
	min_beg1 = IS_sites[2][beg_index+1];
	min_beg2 = IS_sites[3][beg_index+1];
	min_end1 = IS_sites[2][end_index];
	min_end2 = IS_sites[3][end_index];

	if(beg_index > 0 && end_index < IS_num[2] - 1) {
		max_len = IS_sites[2][end_index+1] - IS_sites[2][beg_index];
						
		max_beg1 = IS_sites[2][beg_index]+1;
		max_beg2 = IS_sites[3][beg_index]+1;
		max_end1 = IS_sites[2][end_index+1]-1;
		max_end2 = IS_sites[3][end_index+1]-1;
	}
	else if(beg_index == 0 && end_index < IS_num[2] - 1) {
		max_len = IS_sites[2][end_index+1] - start[0];
						
		max_beg1 = start[0];
		max_beg2 = start[1];
		max_end1 = IS_sites[2][end_index+1]-1;
		max_end2 = IS_sites[3][end_index+1]-1;
	}
	else if(beg_index > 0 && end_index == IS_num[2] - 1) {
		max_len = start[0] + site_end[0] - IS_sites[2][beg_index]-1;
						
		max_beg1 = IS_sites[2][beg_index]+1;
		max_beg2 = IS_sites[3][beg_index]+1;
		max_end1 = start[0] + site_end[0] - 1;
		max_end2 = start[1] + site_end[1] - 1;
	}
	else {
		max_len = site_end[0] + 1;
						
		max_beg1 = start[0];
		max_beg2 = start[1];
		max_end1 = start[0] + site_end[0] - 1;
		max_end2 = start[1] + site_end[1] - 1;
	}
}

char Complement(char c) {
	if(c == 'A')
		return 'T';
	else if(c == 'C')
		return 'G';
	else if(c == 'G')
		return 'C';
	else if(c == 'T')
		return 'A';
	else if(c == 'a')
		return 't';
	else if(c == 'c')
		return 'g';
	else if(c == 'g')
		return 'c';
	else if(c == 't')
		return 'a';
	else if(c == 'N')
		return 'N';
	else if(c == 'n')
		return 'n';
	else if(c == '-')
		return '-';
	else {
		printf("wrong dna sequence : %c\n", c);
		exit(1);
	}
}

/* Check the orthologous sequence in outgroup species is contiguous and long enough. */
int check_orthologs(int start, int end, int ref) {
	int start_net = -1, end_net = -1, i, j, same_chain, chain_num, chain_id[100000], chain_length[100000], max_len, max_len_chain_id, total_net_len, orthologs_start, orthologs_end;

	mismatch_net_num = 0;
	for(i=0; i<net_num[ref-1]; i++) {
		if(net[ref-1][i].end < start)
			continue;
		if(net[ref-1][i].start > end)
			break;
	
		if(start_net == -1)
			start_net = i;
		end_net = i;
	}
	
	if(start_net == -1)
		return 0;

	//check coverage of all nets
	if(net[ref-1][start_net].start > start)
		orthologs_start = net[ref-1][start_net].start;
	else
		orthologs_start = start;
	if(net[ref-1][end_net].end < end)
		orthologs_end = net[ref-1][end_net].end;
	else
		orthologs_end = end;

	if(start_net == end_net) // only one net
		total_net_len = orthologs_end - orthologs_start + 1;
	else {
		total_net_len = net[ref-1][start_net].end - orthologs_start + 1;
		for(i=start_net+1; i<end_net; i++)
			total_net_len += net[ref-1][i].end - net[ref-1][i].start + 1;
		total_net_len += orthologs_end - net[ref-1][end_net].start + 1;
	}
	if((float)total_net_len / (float)(end - start + 1) < MIN_COVERAGE) {
		return 0;
	}
	else if(start_net == end_net) {
		return 1;
	}

	//check if all nets are in the same chain
	same_chain = 1;
	for(i=start_net+1; i<=end_net; i++)
		if(net[ref-1][i].chain_id != net[ref-1][start_net].chain_id)
			same_chain = 0;
	if(same_chain == 1) {
		return 1;
	}
		
	// find majority for all chain
	chain_id[0] = net[ref-1][start_net].chain_id;
	chain_length[0] = net[ref-1][start_net].end - orthologs_start + 1;
	chain_num = 1;
	for(i=start_net+1; i<end_net; i++) {
		for(j=0; j<chain_num; j++)
			if(chain_id[j] == net[ref-1][i].chain_id) {
				chain_length[j] += net[ref-1][i].end - net[ref-1][i].start + 1;
				break;
			}
		if(j == chain_num) {
			chain_id[chain_num] = net[ref-1][i].chain_id;
			chain_length[chain_num] = net[ref-1][i].end - net[ref-1][i].start + 1;
			chain_num++;
		}
	}

	for(j=0; j<chain_num; j++)
		if(chain_id[j] == net[ref-1][end_net].chain_id) {
			chain_length[j] += orthologs_end - net[ref-1][end_net].start + 1;
			break;
		}
	if(j == chain_num) {
		chain_id[chain_num] = net[ref-1][end_net].chain_id;
		chain_length[chain_num] = orthologs_end - net[ref-1][end_net].start + 1;
		chain_num++;
	}

	max_len = chain_length[0];
	max_len_chain_id = chain_id[0];
	for(i=1; i<chain_num; i++) {
		if(chain_length[i] > max_len) {
			max_len = chain_length[i];
			max_len_chain_id = chain_id[i];
		}
	}
	
	//add net into the majority chain without considersing orientation
	int k, l, r_b, l_b;
	for(k=start_net; k<=end_net; k++) {
		if(net[ref-1][k].chain_id == max_len_chain_id)
			break;
	}
	r_b = net[ref-1][k].start2;
	for(i=k-1; i>=start_net; i--) {
		if(strcmp(net[ref-1][i].chr2, net[ref-1][k].chr2) == 0 && net[ref-1][i].orient != net[ref-1][k].orient && net[ref-1][i].end2 < r_b && net[ref-1][i].end2 + MAX_GAP >= r_b) {
			if(i == start_net)
				max_len += net[ref-1][start_net].end - orthologs_start + 1;
			else
				max_len += net[ref-1][i].end - net[ref-1][i].start + 1;
			r_b = net[ref-1][i].start2;
		}
		else {
			mismatch_net[mismatch_net_num] = i;
			mismatch_net_num++;
		}
	}
	
	l = k + 1;
	while(1) {
		while(l <= end_net && net[ref-1][l].chain_id != max_len_chain_id)
			l++;
		if(l > end_net)
			break;
		l_b = net[ref-1][k].end2;
		r_b = net[ref-1][l].start2;
		for(i=k+1; i<l; i++) {
			if(strcmp(net[ref-1][i].chr2, net[ref-1][k].chr2) == 0 && net[ref-1][i].orient != net[ref-1][k].orient && net[ref-1][i].end2 < r_b && net[ref-1][i].start2 > l_b) {
				max_len += net[ref-1][i].end - net[ref-1][i].start + 1;
				l_b = net[ref-1][i].end2;
			}
			else {
				mismatch_net[mismatch_net_num] = i;
				mismatch_net_num++;
			}
		}
		k = l;
		l = l + 1;
	}
	
	l_b = net[ref-1][k].end2;
	for(i=k+1; i<=end_net; i++) {
		if(strcmp(net[ref-1][i].chr2, net[ref-1][k].chr2) == 0 && net[ref-1][i].orient != net[ref-1][k].orient && net[ref-1][i].start2 > l_b && net[ref-1][i].start2 <= l_b + MAX_GAP) {
			if(i == end_net)
				max_len += orthologs_end - net[ref-1][end_net].start + 1;
			else
				max_len += net[ref-1][i].end - net[ref-1][i].start + 1;
			l_b = net[ref-1][i].end2;
		}
		else {
			mismatch_net[mismatch_net_num] = i;
			mismatch_net_num++;
		}
	}
		
	if(((float)max_len / (float)(total_net_len) < MIN_MATCH) || ((float)max_len / (float)(end - start + 1) < MIN_COVERAGE)) {
		return 0;
	}
	else
		return 1;	
}

/* Get orthologous sequence in outgroup species. */
int process_chain(chain_t c, int ref) {
	int i, j, k, old_aln_end1, old_aln_end2;
	char aln_text_temp[1000000], *aln_text_ptr1, *aln_text_ptr2;
	aln_t *aln;
	
	//check orthologs
	if((ref == 1 && !check_orthologs(c.start1, c.end1, ref)) || (ref == 2 && !check_orthologs(c.start2, c.end2, ref)))
		return 0;
	
	if(ref == 1) {
		for(i=0; i<c.end1 - c.start1 + 1; i++) {
			seqs[ref-1][0][i] = '-';
			seqs[ref-1][1][i] = '-';
			seqs[ref-1][2][i] = '-';
		}
	}
	else {
		for(i=0; i<c.end2 - c.start2 + 1; i++) {
			seqs[ref-1][0][i] = '-';
			seqs[ref-1][1][i] = '-';
			seqs[ref-1][2][i] = '-';
		}
	}

	old_aln_end1 = old_aln_end2 = 0; 
	aln = c.aln;
	while(aln != NULL) {
		// remove overlapped part
		aln_text_ptr1 = &(aln->text1[0]);
		aln_text_ptr2 = &(aln->text2[0]);
		if(aln->start1 <= old_aln_end1) {
			aln->len1 -= old_aln_end1 - aln->start1 + 1;
			i = aln->start1;
			while(i <= old_aln_end1) {
				if(*aln_text_ptr1 != '-')
					i++;
				if(*aln_text_ptr2 != '-') {
					aln->start2++;
					aln->len2--;
				}
				aln_text_ptr1++;
				aln_text_ptr2++;
			}
			aln->start1 = old_aln_end1 + 1;
		}
		if(aln->start2 <= old_aln_end2) {
			aln->len2 -= old_aln_end2 - aln->start2 + 1;
			i = aln->start2;
			while(i <= old_aln_end2) {
				if(*aln_text_ptr2 != '-')
					i++;
				if(*aln_text_ptr1 != '-') {
					aln->start1++;
					aln->len1--;
				}
				aln_text_ptr2++;
				aln_text_ptr1++;
			}
			aln->start2 = old_aln_end2 + 1;
		}
		strcpy(aln->text1, aln_text_ptr1);
		strcpy(aln->text2, aln_text_ptr2);
		old_aln_end1 = aln->start1 + aln->len1 - 1;
		old_aln_end2 = aln->start2 + aln->len2 - 1;

		if(ref == 1) {
			for(i=j=k=0; i<aln->len1; i++) {
				while(aln->text1[j] == '-') {
					j++;
					if(aln->text2[j] != '-')
						k++;
				}
				if(seqs[ref-1][0][i + aln->start1+1 - c.start1] == '-') {
					seqs[ref-1][0][i + aln->start1+1 - c.start1] = aln->text1[j];
					seqs[ref-1][1][i + aln->start1+1 - c.start1] = aln->text2[j];
					if(c.orient == '+')
						seq2_pos[ref-1][i + aln->start1+1 - c.start1] = aln->start2 + 1 + k;
					else
						seq2_pos[ref-1][i + aln->start1+1 - c.start1] = aln->src_len2 - aln->start2 - k;
				}
				j++;
				if(aln->text2[j] != '-')
					k++;
			}
		}
		else {
			//reverse and complement sequences if ref = 2
			if(c.orient == '-') {
				strcpy(aln_text_temp, aln->text1);
				for(i=0; i<(int)strlen(aln_text_temp); i++)
					aln->text1[i] = Complement(aln_text_temp[strlen(aln_text_temp) - i - 1]);
				aln->start1 = aln->src_len1 - (aln->start1 + aln->len1);
				strcpy(aln_text_temp, aln->text2);
				for(i=0; i<(int)strlen(aln_text_temp); i++)
					aln->text2[i] = Complement(aln_text_temp[strlen(aln_text_temp) - i - 1]);
				aln->start2 = aln->src_len2 - (aln->start2 + aln->len2);
			}
			for(i=j=k=0; i<aln->len2; i++) {
				while(aln->text2[j] == '-') {
					j++;
					if(aln->text1[j] != '-')
						k++;
				}
				if(seqs[ref-1][0][i + aln->start2+1 - c.start2] == '-') {
					seqs[ref-1][0][i + aln->start2+1 - c.start2] = aln->text2[j];
					seqs[ref-1][1][i + aln->start2+1 - c.start2] = aln->text1[j];
					if(c.orient == '+')
						seq2_pos[ref-1][i + aln->start2+1 - c.start2] = aln->start1 + 1 + k;
					else 					
						seq2_pos[ref-1][i + aln->start2+1 - c.start2] = aln->src_len1 - aln->start1 - k;
				}
				j++;
				if(aln->text1[j] != '-')
					k++;
			}
		}
		
		aln = aln->next;
	}

	//find orthologous sequence
	for(i=0; i<net_num[ref-1]; i++) {
		if(net[ref-1][i].end < start[ref-1])
			continue;
		if(net[ref-1][i].start > end[ref-1])
			break;
				
		for(j=0; j<mismatch_net_num; j++)
			if(mismatch_net[j] == i)
				break;
		if(j<mismatch_net_num) {
			continue;
		}
		
		if(rescored[ref-1][i] == 0) {
			rescoring(ref, i);
			rescored[ref-1][i] = 1;
		}
			
		k = 0;
		for(j=net[ref-1][i].start; j<=net[ref-1][i].end; j++) {
			while(net[ref-1][i].net1[k] == '-')
				k++;
			if(j >= start[ref-1] && j <= end[ref-1]) {
				if(net[ref-1][i].net1[k] != 'X')
					seqs[ref-1][2][j - start[ref-1]] = net[ref-1][i].net2[k];
			}
			else if(j > end[ref-1]) {
				break;
			}
			k++;
		}
	}
	
	return 1;
}

void free_aln(aln_t *aln) {
	if(aln->next != NULL)
		free_aln(aln->next);
	
	free(aln);

	return;
}


/* Calculate the starting and ending positions for each sub-alignment */
int aln2_pos(chain_t c, int pos1) {
	aln_t *aln;
	int i, j, pos2;
	
	aln = c.aln;
	while(aln != NULL) {
		if(pos1 < aln->start1 + aln->len1) {
			j = aln->start1 - 1;
			if(c.orient == '+') {
				pos2 = aln->start2 - 1;
				for(i=0; j<pos1; i++) {
					if(aln->text1[i] != '-')
						j++;
					if(aln->text2[i] != '-')
						pos2++;
				}
			}
			else {
				pos2 = aln->src_len2 - aln->start2 + 1;
				for(i=0; j<pos1; i++) {
					if(aln->text1[i] != '-')
						j++;
					if(aln->text2[i] != '-')
						pos2--;
				}
			}
			return pos2;
		}
		else if(pos1 < aln->next->start1) {
			if(c.orient == '+')
				pos2 = aln->start2 + aln->len2 + (pos1 - (aln->start1 + aln->len1));
			else
				pos2 = aln->src_len2 - (aln->start2 + aln->len2) - (pos1 - (aln->start1 + aln->len1));
			return pos2;
		}
		
		aln = aln->next;
	}
	
	return -1;
}

/* Long alignments are divided into several sub-alignments with 1000 sites overlapped. */
void divided_index(chain_t c) {
	int len;
	
	len = c.end1 - c.start1 + 1;
	if(len <= 5000) {
		d_beg_index[0] = c.start2;
		d_end_index[0] = c.end2;
		d_index_num = 1;
	}
	else {
		d_index_num = 0;
		site_beg[0] = 0;
		site_end[0] = 4999;
		while(site_end[0] < len) {
			d_beg_index[d_index_num] = aln2_pos(c, site_beg[0] + c.start1 - 1);
			d_end_index[d_index_num] = aln2_pos(c, site_end[0] + c.start1 - 1);
			d_index_num++;
			if(site_end[0] == len - 1)
				break;
			else if(site_end[0] + 4000 >= len - 1) {
				site_beg[0] = len - 5000;
				site_end[0] = len - 1  ;
			}
			else {
				site_beg[0] = site_end[0] - 1000;
				site_end[0] = site_end[0] + 4000;
			}
		}
	}
}

/* Determine the directionality of a gene conversion event. */
float determine_directionality() {
	int md_m1, md_n1, md_m2, md_n2, i;
	float p, p1, p2, V, T;
	
	md_m1 = md_n1 = md_m2 = md_n2 = 0;
	for(i=beg_index+1; i<=end_index; i++) {
		if(IS_ref[i] == 0) {
			if(IS[2][i] == 1)
				md_m1++;
			else if(IS[2][i] == 2)
				md_n1++;
		}
		else if(IS_ref[i] == 1) {
			if(IS[2][i] == 1)
				md_m2++;
			else if(IS[2][i] == 2)
				md_n2++;
		}
	}

	p = (float)(md_n1 + md_n2) / (float)(md_m1 + md_m2 + md_n1 + md_n2);
	p1 = (float)md_n1 / (float)(md_m1 + md_n1);
	p2 = (float)md_n2 / (float)(md_m2 + md_n2);
	V = ((1.0/(float)(md_m1 + md_n1)) + (1.0/(float)(md_m2 + md_n2))) * p * (1.0 - p);
	if(V == 0.0 || (md_m1 + md_n1) == 0 || (md_m2 + md_n2) == 0)
		T = 0.0;
	else
		T = ((float)(p1 - p2)) / sqrtf(V);
	
	return T;
}

/* Read paralogous pairs and output m, n and k. */
int read_chain(char *chain_fn) {
	FILE *fp;
	char *not_eof;
	chain_t c;
	int index;
	aln_t **aln_ptr, *aln;
	int i, match, mismatch, triplet_status, aln_len;
	float d_conf;

	fp = fopen(chain_fn, "r");
	while((not_eof = fgets(buf, 1000000, fp)) && buf[0] == '#')
		;
	while(not_eof) {
		sscanf(buf, "%d %s %d %d %s %d %d %c", &index, c.chr1, &c.start1, &c.end1, c.chr2, &c.start2, &c.end2, &c.orient);
		// test length
		if((c.end1 - c.start1 + 1) > 1000000 || (c.end2 - c.start2 + 1) > 1000000) {
			printf("chain is too long : %d - %d\n", c.end1 - c.start1 + 1, c.end2 - c.start2 + 1);
			exit(1);
		}
		fgets(buf, 1000000, fp);
		aln_ptr = &c.aln;
		while(buf[0] == 'a') {
			*aln_ptr = (aln_t *)malloc(sizeof(aln_t));
			fgets(buf, 1000000, fp);
			sscanf(buf, "s %s %d %d %c %d %s", c.chr1, &(*aln_ptr)->start1, &(*aln_ptr)->len1, &c.orient, &(*aln_ptr)->src_len1, (*aln_ptr)->text1);
			fgets(buf, 1000000, fp);
			sscanf(buf, "s %s %d %d %c %d %s", c.chr2, &(*aln_ptr)->start2, &(*aln_ptr)->len2, &c.orient, &(*aln_ptr)->src_len2, (*aln_ptr)->text2);
			(*aln_ptr)->next = NULL;
			aln_ptr = &((*aln_ptr)->next);
			fgets(buf, 1000000, fp);
			not_eof = fgets(buf, 1000000, fp);
		}
		
		start[0] = c.start1;
		end[0] = c.end1;
		sites_num[0] = c.end1 - c.start1 + 1;
		start[1] = c.start2;
		end[1] = c.end2;
		sites_num[1] = c.end2 - c.start2 + 1;
		
		//calculate identity
		aln = c.aln;
		match = mismatch = 0;
		while(aln != NULL) {
			aln_len = strlen(aln->text1);
			for(i=0; i<aln_len; i++) {
				if(toupper(aln->text1[i]) == toupper(aln->text2[i]))
					match++;
				else
					mismatch++;
			}
			aln = aln->next;
		}
		identity = (double)match / (double)(match + mismatch);
		
		divided_index(c);

		site_beg[0] = 0;
		site_beg[1] = 0;
		long_seqs = 0;
		if(process_chain(c, 1) == 1) {	
			if(process_chain(c, 2) == 1) 
				triplet_status = 3;
			else 
				triplet_status = 1;
		}		
		else if(process_chain(c, 2) == 1) 		
			triplet_status = 2;
		else
			triplet_status = 0;
		
		// test conversion ratio
		site_end[0] = sites_num[0];
		if(triplet_status == 1 || triplet_status == 3) {
			find_IS(1);
			find_max_descent(1);
		}
		else {
			IS_num[0] = 0;
			m[0] = 0;
			n[0] = 0;
			max_descent[0] = 0;
		}
		site_end[1] = sites_num[1];
		if(triplet_status == 2 || triplet_status == 3) {
			find_IS(2);
			find_max_descent(2);
		}
		else {
			IS_num[1] = 0;
			m[1] = 0;
			n[1] = 0;
			max_descent[1] = 0;
		}
		d_conf = 0.0;
		if(triplet_status != 0) {
			combine_IS(c.orient);
			find_max_descent(3);
			if(triplet_status == 3)
				d_conf = determine_directionality();
		}
		if(triplet_status == 0 || (float)(abs(max_end1 - max_beg1) + 1) / (float)sites_num[0] > MAX_GC_RATIO) {		// Orthologs are missing or possible conversion regions are too long.
			printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\n", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], 0, 0, 0, 0, 0, 0, 0, 0, 0, identity, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, d_conf);	
			for(i=5000; i<sites_num[0]; i+=4000)
				printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\n", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], 0, 0, 0, 0, 0, 0, 0, 0, 0, identity, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, d_conf);					
		}
		else if(sites_num[0] <= 5000)	// Short alignments
			printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\n", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], m[0], n[0], max_descent[0], m[1], n[1], max_descent[1], m[0] + m[1], n[0] + n[1], max_descent[2], identity, min_len, max_len, max_beg1, min_beg1, min_end1, max_end1, max_beg2, min_beg2, min_end2, max_end2, d_conf);
		else {	// Long alignments
			long_seqs = 1;
			site_end[0] = 5000;
			i = 0;
			while(site_end[0] <= sites_num[0]) {
				if(triplet_status == 1 || triplet_status == 3) {
					find_IS(1);
					find_max_descent(1);
				}
				else {
					IS_num[0] = 0;
					m[0] = 0;
					n[0] = 0;
					max_descent[0] = 0;
				}
				if(triplet_status == 2 || triplet_status == 3) {
					if(c.orient == '+') {
						site_beg[1] = d_beg_index[i] - c.start2;
						site_end[1] = d_end_index[i] - c.start2;
					}
					else {
						site_beg[1] = d_end_index[i] - c.start2;
						site_end[1] = d_beg_index[i] - c.start2;
					}
					find_IS(2);
					find_max_descent(2);
				}
				else {
					IS_num[1] = 0;
					m[1] = 0;
					n[1] = 0;
					max_descent[1] = 0;
				}
				combine_IS(c.orient);
				find_max_descent(3);
				printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\n", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], m[0], n[0], max_descent[0], m[1], n[1], max_descent[1], m[0] + m[1], n[0] + n[1], max_descent[2], identity, min_len, max_len, max_beg1, min_beg1, min_end1, max_end1, max_beg2, min_beg2, min_end2, max_end2, d_conf);
				
				if(site_end[0] == sites_num[0])
					break;
				else if(site_end[0] + 4000 >= sites_num[0]) {
					site_beg[0] = sites_num[0] - 5000;
					site_end[0] = sites_num[0];
				}
				else {
					site_beg[0] = site_end[0] - 1000;
					site_end[0] = site_end[0] + 4000;
				}
				i++;
			}
		}	
		
		//free aln
		free_aln(c.aln);
	}

	fclose(fp);
	
	return 0;
}

int main(int argc, char *argv[]) {
	if ( argc != 6 && argc != 7) {
		printf("md_quadruplet_rescoring chain-file net_file axt_file net_file axt_file <score-file> \n");
		return 1;
	}
	// read input file
	if(argc == 7)
		read_score(argv[6]);
	read_net(argv[2], argv[3], 1);
	read_net(argv[4], argv[5], 2);
	read_chain(argv[1]);

	return 0;
}
