#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "maf.h"

#define MAX_GAP 5000
#define MIN_COVERAGE 0.5
#define MIN_MATCH 0.8	
#define MAX_GC_RATIO 0.8

typedef struct Net {
	int start, end, start2, end2, block_index;
	char *net1, *net2, chr2[100], orient;
} net_t;

typedef struct Net_level {
	int chain_id, end, chr_size;
	char chr2[100];
} net_level_t;

typedef struct Aln {
	int start1, len1, src_len1, start2, len2, src_len2;
	char text1[1000000], text2[1000000];
	struct Aln *next;
} aln_t;

typedef struct Chain {
	char chr1[100], chr2[100], orient;
	int start1, end1, start2, end2;
	aln_t *aln;
} chain_t;

net_level_t net_level[1000];
int net_level_num;
net_t net[1][1000000];
int rescored[1][1000000];
int net_num[1];
char buf[1000000];
char seqs[2][3][1000000];
int seq2_pos[2][1000000];
int d_beg_index[10000], d_end_index[10000], d_index_num;
int sites_num[2];
int IS[3][1000000], IS_num[3], IS_sites[4][1000000], IS_ref[1000000];
int IS_quadruplet[1000000], IS_quadruplet_num, IS_quadruplet_sites[2][1000000];
int m[2], n[2], max_descent[3], beg_index = 0, end_index = 0;
double identity;
int max_len, min_len, max_beg1, min_beg1, min_end1, max_end1, max_beg2, min_beg2, min_end2, max_end2, start[2], end[2];
int offset[3];
int site_beg[2], site_end[2];
int mismatch_net[100000];
int mismatch_net_num;
int long_seqs;
//score parameters
int if_rescoring = 0;
int fill_score = -100, T = 1, Z = 1, O = 400, E = 30, X = 910, Y = 9400, K = 3000, L = 3000; 
int score[4][4] = {{91, -114, -31, -123}, {-114, 100, -125, -31}, {-31, -125, 100, -114}, {-123, -31, -114, 91}};
char outgroup_name[2][100], outgroup_orient[2];
int outgroup_start[2], outgroup_end[2], orthologs_block[2][100], orthologs_block_num[2];

void read_score(char *filename) {
	FILE *fp;
	char buf[10000], *temp_str;
	
	fp = fopen(filename, "r");
	while(fgets(buf, 10000, fp)) {
		if(buf[0] == '#')
			continue;
		if(strncmp(buf, "fill_score", 10) == 0) {
			strtok(buf, "=");
			temp_str = strtok(NULL, "=");
			sscanf(temp_str, "%d", &fill_score);
		}
		else if (buf[1] == '=') {
			if(buf[0] == 'T') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &T);
			}
			else if(buf[0] == 'O') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &O);
			}
			else if(buf[0] == 'E') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &E);
			}
			else if(buf[0] == 'X') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &X);
			}
			else if(buf[0] == 'Y') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &Y);
			}
			else if(buf[0] == 'K') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &K);
			}
			else if(buf[0] == 'L') {
				strtok(buf, "=");
				temp_str = strtok(NULL, "=");
				sscanf(temp_str, "%d", &L);
			}
		}
		else if(buf[0] == 'A')
			sscanf(buf, "A %d %d %d %d", &score[0][0], &score[0][1], &score[0][2], &score[0][3]);
		else if(buf[0] == 'C')
			sscanf(buf, "C %d %d %d %d", &score[1][0], &score[1][1], &score[1][2], &score[1][3]);
		else if(buf[0] == 'G')
			sscanf(buf, "G %d %d %d %d", &score[2][0], &score[2][1], &score[2][2], &score[2][3]);
		else if(buf[0] == 'T')
			sscanf(buf, "T %d %d %d %d", &score[3][0], &score[3][1], &score[3][2], &score[3][3]);
	}
	/*
	printf("fill_score = %d; T = %d; O = %d; E = %d; X = %d; Y = %d; K = %d; L = %d\n", fill_score, T, O, E, X, Y, K, L);
	for(i=0; i<4; i++) {
		for(j=0; j<4; j++)
			printf("%d\t", score[i][j]);
		printf("\n");
	}
	*/
}

int cal_score(char A, char B, int gap_opend) {
	if(A == '-' || B == '-') {
		if(gap_opend == 1)
			return -E;
		else
			return -O;
	}
	if(toupper(A) == 'A') {
		if(toupper(B) == 'A')
			return score[0][0];
		else if(toupper(B) == 'C')
			return score[0][1];
		else if(toupper(B) == 'G')
			return score[0][2];
		else if(toupper(B) == 'T')
			return score[0][3];
		else
			return fill_score;
	}
	else if(toupper(A) == 'C') {
		if(toupper(B) == 'A')
			return score[1][0];
		else if(toupper(B) == 'C')
			return score[1][1];
		else if(toupper(B) == 'G')
			return score[1][2];
		else if(toupper(B) == 'T')
			return score[1][3];
		else
			return fill_score;
	}
	else if(toupper(A) == 'G') {
		if(toupper(B) == 'A')
			return score[2][0];
		else if(toupper(B) == 'C')
			return score[2][1];
		else if(toupper(B) == 'G')
			return score[2][2];
		else if(toupper(B) == 'T')
			return score[2][3];
		else
			return fill_score;
	}
	else if(toupper(A) == 'T') {
		if(toupper(B) == 'A')
			return score[3][0];
		else if(toupper(B) == 'C')
			return score[3][1];
		else if(toupper(B) == 'G')
			return score[3][2];
		else if(toupper(B) == 'T')
			return score[3][3];
		else
			return fill_score;
	}
	else
		return fill_score;
}

void rescoring(int i) {
	int j, k, max_score, cur_score, gap_opend, aln_beg, aln_max, gap_free, net_len;

	max_score = cur_score = gap_opend = aln_beg = aln_max = 0;
	gap_free = 1;
	net_len = strlen(net[0][i].net1);
	for(j=0; j<net_len; j++) {
		cur_score += cal_score(net[0][i].net1[j], net[0][i].net2[j], gap_opend);	
		if(net[0][i].net1[j] == '-' || net[0][i].net2[j] == '-') {
			gap_opend = 1;
			gap_free = 0;
		}
		else
			gap_opend = 0;
				
		if(cur_score >= max_score) {
			max_score = cur_score;
			aln_max = j;
		}

		//if(cur_score < 0 || (gap_free == 1 && (max_score - cur_score) > X) || (gap_free == 0 && (max_score - cur_score) > Y) || j == net_len - 1) { 
		if(cur_score < 0 || (max_score - cur_score) > Y || j == net_len - 1) {
			if(max_score < L) //throw away whole segment
				for(k=aln_beg; k<=j; k++) {
					if(net[0][i].net1[k] != '-')
						net[0][i].net1[k] = 'X';
					if(net[0][i].net2[k] != '-')
						net[0][i].net2[k] = 'X';
				}
			else {	//throw away the dropping region
				for(k=aln_max+1; k<=j; k++) {
					if(net[0][i].net1[k] != '-')
						net[0][i].net1[k] = 'X';
					if(net[0][i].net2[k] != '-')
						net[0][i].net2[k] = 'X';
				}
			}
			aln_beg = aln_max = j + 1;
			max_score = cur_score = 0;
			gap_free = 1;
		}
	}
	
	return;
}

int sort_net(void* a, void* b) {
	return (((net_t*)a)->start - ((net_t*)b)->start);
}

int read_orthologs(char filename[1000]) {
	struct mafFile *mf;
	struct mafAli *ali;
	struct mafComp *mc1, *mc2;
	
	net_num[0] = 0;
	mf = mafOpen(filename, 0);
	while((ali = mafNext(mf)) != NULL) {
		mc1 = ali->components;
		if(mc1 == NULL) {
			printf("Wrong orthologs file\n");
			exit(1);
		}
		mc2 = mc1->next;
		if(mc2 == NULL) {
			printf("Wrong orthologs file\n");
			exit(1);
		}
		net[0][net_num[0]].block_index = ali->score;
		net[0][net_num[0]].start = mc1->start + 1;
		net[0][net_num[0]].end = mc1->start + mc1->size;
		if(mc2->strand == '+') {
			net[0][net_num[0]].start2 = mc2->start + 1;
			net[0][net_num[0]].end2 = mc2->start + mc2->size;
		}
		else {
			net[0][net_num[0]].start2 = mc2->srcSize - mc2->start - mc2->size + 1;
			net[0][net_num[0]].end2 = mc2->srcSize - mc2->start;
		}
		strcpy(net[0][net_num[0]].chr2, mc2->name);
		net[0][net_num[0]].orient = mc2->strand;
		net[0][net_num[0]].net1 = (char *)malloc(sizeof(char) * (strlen(mc1->text) + 1));
		strcpy(net[0][net_num[0]].net1, mc1->text);
		net[0][net_num[0]].net2 = (char *)malloc(sizeof(char) * (strlen(mc2->text) + 1));
		strcpy(net[0][net_num[0]].net2, mc2->text);
		net_num[0]++;
	}
	
	qsort((void*)net[0], net_num[0], sizeof(net_t), (void *)sort_net);
	/*
	printf("net_num = %d\n", net_num[0]);
	int i;
	for(i=0; i<net_num[0]; i++) 
		printf("%d\t%d\n", net[0][i].start, net[0][i].end);
	*/
	return 0;
}

void find_IS(int ref) {
	int i;
	
	IS_num[ref-1] = 0;
	/*
	for(j=0; j<3; j++) {
		for(i=0; i<sites_num; i++)
			printf("%c", seqs[j][i]);
		printf("\n\n");
	}
	*/
	
	m[ref-1] = 0;
	n[ref-1] = 0;
	
	for(i=site_beg[ref-1]; i<site_end[ref-1]; i++) {
		//there are gaps in a site
		if((seqs[ref-1][0][i] == '-') || (seqs[ref-1][1][i] == '-') || (seqs[ref-1][2][i] == '-') || (seqs[ref-1][0][i] == 'N') || (seqs[ref-1][1][i] == 'N') || (seqs[ref-1][2][i] == 'N') || (seqs[ref-1][0][i] == 'n') || (seqs[ref-1][1][i] == 'n') || (seqs[ref-1][2][i] == 'n'))
			continue;
		//all are the same in a site
		if((toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][1][i])) && (toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][2][i])))
			continue;
		// all are different in a site
		else if((toupper(seqs[ref-1][0][i]) != toupper(seqs[ref-1][1][i])) && (toupper(seqs[ref-1][0][i]) != toupper(seqs[ref-1][2][i])) && (toupper(seqs[ref-1][1][i]) != toupper(seqs[ref-1][2][i])))
			continue;
		//type P
		else if(toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][2][i])) {
			IS[ref-1][IS_num[ref-1]] = 1;
			m[ref-1]++;
		}
		//type Q	
		else if(toupper(seqs[ref-1][0][i]) == toupper(seqs[ref-1][1][i])) {
			IS[ref-1][IS_num[ref-1]] = 2;
			n[ref-1]++;
		}
		else
			continue;
		
		//printf("%d\n", i+1);
		IS_sites[ref-1][IS_num[ref-1]] = i;
		IS_num[ref-1]++;
	}
	
	/*
	int walk;
	walk = 0;
	for(i=IS_num[ref-1]-1; i>=0; i--) {
		if(IS[ref-1][i] == 1)
			walk++;
		else
			walk--;
		printf("%d\t%d\n", site_end[ref-1] - IS_sites[ref-1][i] - 1, walk);
	}
	
	printf("\n\n");
	
	for(i=0; i<IS_num[ref-1]; i++)
		printf("%d", IS[ref-1][i]);
	printf("\n\n");
	*/
}

void combine_IS(char orient) {
	int IS_index[2], i;

	if(orient == '+') {
		IS_index[0] = IS_index[1] = 0;
		IS_num[2] = 0;
		while(IS_index[0] < IS_num[0] || IS_index[1] < IS_num[1]) {
			//printf("%d\t%d\n", IS_index[0], IS_index[1]);
			if(IS_index[0] >= IS_num[0]) {
				for(i=IS_index[1]; i<IS_num[1]; i++) {
					IS[2][IS_num[2]] = IS[1][i];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][i]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][i];
					IS_num[2]++;
				}
				break;
			}
			else if(IS_index[1] >= IS_num[1]) {
				for(i=IS_index[0]; i<IS_num[0]; i++) {
					IS[2][IS_num[2]] = IS[0][i];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][i];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][i]];
					IS_num[2]++;
				}
				break;
			}
			else {
				if((start[0] + IS_sites[0][IS_index[0]]) < seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_index[0]++;
				}
				else if ((start[0] + IS_sites[0][IS_index[0]]) > seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[1]++;
				}
				else {	
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_num[2]++;
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[0]++;
					IS_index[1]++;
				}
				IS_num[2]++;
			}   
		}
	} 
	else {
		IS_index[0] = 0;
		IS_index[1] = IS_num[1] - 1;
		IS_num[2] = 0;
		while(IS_index[0] < IS_num[0] || IS_index[1] >= 0) {
			//printf("%d\t%d\n", IS_index[0], IS_index[1]);
			if(IS_index[0] >= IS_num[0]) {
				for(i=IS_index[1]; i>=0; i--) {
					IS[2][IS_num[2]] = IS[1][i];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][i]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][i];
					IS_num[2]++;
				}
				break;
			}
			else if(IS_index[1] < 0) {
				for(i=IS_index[0]; i<IS_num[0]; i++) {
					IS[2][IS_num[2]] = IS[0][i];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][i];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][i]];
					IS_num[2]++;
				}
				break;
			}
			else {
				if((start[0] + IS_sites[0][IS_index[0]]) < seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_index[0]++;
				}
				else if ((start[0] + IS_sites[0][IS_index[0]]) > seq2_pos[1][IS_sites[1][IS_index[1]]]) {
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[1]--;
				}
				else {
					IS[2][IS_num[2]] = IS[0][IS_index[0]];
					IS_ref[IS_num[2]] = 0;
					IS_sites[2][IS_num[2]] = start[0] + IS_sites[0][IS_index[0]];
					IS_sites[3][IS_num[2]] = seq2_pos[0][IS_sites[0][IS_index[0]]];
					IS_num[2]++;
					IS[2][IS_num[2]] = IS[1][IS_index[1]];
					IS_ref[IS_num[2]] = 1;
					IS_sites[2][IS_num[2]] = seq2_pos[1][IS_sites[1][IS_index[1]]];
					IS_sites[3][IS_num[2]] = start[1] + IS_sites[1][IS_index[1]];
					IS_index[0]++;
					IS_index[1]--;
				}
				IS_num[2]++;
			}   
		}
	}
	/*
	for(i=0; i<IS_num[2]; i++)
		printf("%d", IS[2][i]);
	printf("\n\n");
	*/
}

void find_max_descent(int ref) {
	int max_height = 0, height = 0, descent = 0, i, max_height_index = 0;
	
	max_descent[ref-1] = 0;
	beg_index = end_index = 0;
	for(i=0; i<IS_num[ref-1]; i++) {
		if(IS[ref-1][i] == 1) {
			height++;
			if( height > max_height ) { 
				max_height = height;
				max_height_index = i;
			}
			descent = max_height - height;
			if( descent >= max_descent[ref-1] ) { 
				max_descent[ref-1] = descent;
				beg_index = max_height_index;
				end_index = i;
			}
		}
		else if(IS[ref-1][i] == 2) {
			height--;
			if( height > max_height ) { 
				max_height = height;
				max_height_index = i;
			}
			descent = max_height - height;
			if( descent >= max_descent[ref-1] ) {
				max_descent[ref-1] = descent;
				beg_index = max_height_index;
				end_index = i;
			}
		}
	}
	
	if(long_seqs == 1 || ref < 3)
		return;
	
	min_len = max_len = 0;
	/*
	for(i=IS_sites[beg_index+1]; i<=IS_sites[end_index]; i++)
			if(seqs[0][i] != '-')
				min_len++;
	*/
	min_len = IS_sites[2][end_index] - IS_sites[2][beg_index+1] + 1;
	min_beg1 = IS_sites[2][beg_index+1];
	min_beg2 = IS_sites[3][beg_index+1];
	min_end1 = IS_sites[2][end_index];
	min_end2 = IS_sites[3][end_index];

	if(beg_index > 0 && end_index < IS_num[2] - 1) {
		/*
		for(i=IS_sites[beg_index]+1; i<IS_sites[end_index+1]; i++)
			if(seqs[0][i] != '-')
				max_len++;
		*/
		max_len = IS_sites[2][end_index+1] - IS_sites[2][beg_index];
						
		max_beg1 = IS_sites[2][beg_index]+1;
		max_beg2 = IS_sites[3][beg_index]+1;
		max_end1 = IS_sites[2][end_index+1]-1;
		max_end2 = IS_sites[3][end_index+1]-1;
	}
	else if(beg_index == 0 && end_index < IS_num[2] - 1) {
		/*
		for(i=0; i<IS_sites[end_index+1]; i++)
			if(seqs[0][i] != '-')
				max_len++;
		*/
		max_len = IS_sites[2][end_index+1] - start[0];
						
		max_beg1 = start[0];
		max_beg2 = start[1];
		max_end1 = IS_sites[2][end_index+1]-1;
		max_end2 = IS_sites[3][end_index+1]-1;
	}
	else if(beg_index > 0 && end_index == IS_num[2] - 1) {
		/*
		for(i=IS_sites[beg_index]+1; i<sites_num; i++)
			if(seqs[0][i] != '-')
				max_len++;
		*/
		max_len = start[0] + site_end[0] - IS_sites[2][beg_index]-1;
						
		max_beg1 = IS_sites[2][beg_index]+1;
		max_beg2 = IS_sites[3][beg_index]+1;
		max_end1 = start[0] + site_end[0] - 1;
		max_end2 = start[1] + site_end[1] - 1;
	}
	else {
		/*
		for(i=0; i<sites_num; i++)
			if(seqs[0][i] != '-')
				max_len++;
		*/
		max_len = site_end[0] + 1;
						
		max_beg1 = start[0];
		max_beg2 = start[1];
		max_end1 = start[0] + site_end[0] - 1;
		max_end2 = start[1] + site_end[1] - 1;
	}
}

char Complement(char c) {
	if(c == 'A')
		return 'T';
	else if(c == 'C')
		return 'G';
	else if(c == 'G')
		return 'C';
	else if(c == 'T')
		return 'A';
	else if(c == 'a')
		return 't';
	else if(c == 'c')
		return 'g';
	else if(c == 'g')
		return 'c';
	else if(c == 't')
		return 'a';
	else if(c == 'N')
		return 'N';
	else if(c == 'n')
		return 'n';
	else if(c == '-')
		return '-';
	else {
		printf("wrong dna sequence : %c\n", c);
		exit(1);
	}
}

int check_orthologs(int start, int end) {
	int start_net = -1, end_net = -1, i, total_net_len, orthologs_start, orthologs_end;

	mismatch_net_num = 0;
	for(i=0; i<net_num[0]; i++) {
		if(net[0][i].end < start)
			continue;
		if(net[0][i].start > end)
			break;
	
		if(start_net == -1)
			start_net = i;
		end_net = i;
	}
	
	if(start_net == -1)
		return 0;

	//check coverage of all nets
	if(net[0][start_net].start > start)
		orthologs_start = net[0][start_net].start;
	else
		orthologs_start = start;
	if(net[0][end_net].end < end)
		orthologs_end = net[0][end_net].end;
	else
		orthologs_end = end;

	if(start_net == end_net) // only one net
		total_net_len = orthologs_end - orthologs_start + 1;
	else {
		total_net_len = net[0][start_net].end - orthologs_start + 1;
		for(i=start_net+1; i<end_net; i++)
			total_net_len += net[0][i].end - net[0][i].start + 1;
		total_net_len += orthologs_end - net[0][end_net].start + 1;
	}
	if((float)total_net_len / (float)(end - start + 1) < MIN_COVERAGE) {
		return 0;
	}
	else if(start_net == end_net) {
		return 1;
	}

	return 1;	
}

int process_chain(chain_t c, int ref) {
	int i, j, k, l, old_aln_end1, old_aln_end2;
	char aln_text_temp[1000000], *aln_text_ptr1, *aln_text_ptr2;
	aln_t *aln;
	
	//check orthologs
	if((ref == 1 && !check_orthologs(c.start1, c.end1)) || (ref == 2 && !check_orthologs(c.start2, c.end2)))
		return 0;
	
	if(ref == 1)
		for(i=0; i<c.end1 - c.start1 + 1; i++) {
			seqs[ref-1][0][i] = '-';
			seqs[ref-1][1][i] = '-';
			seqs[ref-1][2][i] = '-';
		}
	else
		for(i=0; i<c.end2 - c.start2 + 1; i++) {
			seqs[ref-1][0][i] = '-';
			seqs[ref-1][1][i] = '-';
			seqs[ref-1][2][i] = '-';
		}

	old_aln_end1 = old_aln_end2 = 0; 
	aln = c.aln;
	while(aln != NULL) {
		// remove overlapped part
		aln_text_ptr1 = &(aln->text1[0]);
		aln_text_ptr2 = &(aln->text2[0]);
		if(aln->start1 <= old_aln_end1) {
			aln->len1 -= old_aln_end1 - aln->start1 + 1;
			i = aln->start1;
			while(i <= old_aln_end1) {
				if(*aln_text_ptr1 != '-')
					i++;
				if(*aln_text_ptr2 != '-') {
					aln->start2++;
					aln->len2--;
				}
				aln_text_ptr1++;
				aln_text_ptr2++;
			}
			aln->start1 = old_aln_end1 + 1;
		}
		if(aln->start2 <= old_aln_end2) {
			aln->len2 -= old_aln_end2 - aln->start2 + 1;
			i = aln->start2;
			while(i <= old_aln_end2) {
				if(*aln_text_ptr2 != '-')
					i++;
				if(*aln_text_ptr1 != '-') {
					aln->start1++;
					aln->len1--;
				}
				aln_text_ptr2++;
				aln_text_ptr1++;
			}
			aln->start2 = old_aln_end2 + 1;
		}
		strcpy(aln->text1, aln_text_ptr1);
		strcpy(aln->text2, aln_text_ptr2);
		old_aln_end1 = aln->start1 + aln->len1 - 1;
		old_aln_end2 = aln->start2 + aln->len2 - 1;

		if(ref == 1) {
			for(i=j=k=0; i<aln->len1; i++) {
				while(aln->text1[j] == '-') {
					j++;
					if(aln->text2[j] != '-')
						k++;
				}
				if(seqs[ref-1][0][i + aln->start1+1 - c.start1] == '-') {
					seqs[ref-1][0][i + aln->start1+1 - c.start1] = aln->text1[j];
					seqs[ref-1][1][i + aln->start1+1 - c.start1] = aln->text2[j];
					if(c.orient == '+')
						seq2_pos[ref-1][i + aln->start1+1 - c.start1] = aln->start2 + 1 + k;
					else
						seq2_pos[ref-1][i + aln->start1+1 - c.start1] = aln->src_len2 - aln->start2 - k;
				}
				j++;
				if(aln->text2[j] != '-')
					k++;
			}
		}
		else {
			//reverse and complement sequences if ref = 2
			if(c.orient == '-') {
				strcpy(aln_text_temp, aln->text1);
				for(i=0; i<(int)strlen(aln_text_temp); i++)
					aln->text1[i] = Complement(aln_text_temp[strlen(aln_text_temp) - i - 1]);
				aln->start1 = aln->src_len1 - (aln->start1 + aln->len1);
				strcpy(aln_text_temp, aln->text2);
				for(i=0; i<(int)strlen(aln_text_temp); i++)
					aln->text2[i] = Complement(aln_text_temp[strlen(aln_text_temp) - i - 1]);
				aln->start2 = aln->src_len2 - (aln->start2 + aln->len2);
			}
			for(i=j=k=0; i<aln->len2; i++) {
				while(aln->text2[j] == '-') {
					j++;
					if(aln->text1[j] != '-')
						k++;
				}
				if(seqs[ref-1][0][i + aln->start2+1 - c.start2] == '-') {
					seqs[ref-1][0][i + aln->start2+1 - c.start2] = aln->text2[j];
					seqs[ref-1][1][i + aln->start2+1 - c.start2] = aln->text1[j];
					if(c.orient == '+')
						seq2_pos[ref-1][i + aln->start2+1 - c.start2] = aln->start1 + 1 + k;
					else 					
						seq2_pos[ref-1][i + aln->start2+1 - c.start2] = aln->src_len1 - aln->start1 - k;
				}
				j++;
				if(aln->text1[j] != '-')
					k++;
			}
		}
		
		aln = aln->next;
	}

	//find orthologous sequence
	for(i=0; i<net_num[0]; i++) {
		if(net[0][i].end < start[ref-1])
			continue;
		if(net[0][i].start > end[ref-1])
			break;
				
		for(j=0; j<mismatch_net_num; j++)
			if(mismatch_net[j] == i)
				break;
		if(j<mismatch_net_num) {
			//printf("%d\n", i);
			continue;
		}
		
		orthologs_block[ref-1][orthologs_block_num[ref-1]] = net[0][i].block_index;
		orthologs_block_num[ref-1]++;
		
		if(if_rescoring && rescored[0][i] == 0) {
			rescoring(i);
			rescored[0][i] = 1;
		}

		strcpy(outgroup_name[ref-1], net[0][i].chr2);
		outgroup_orient[ref-1] = net[0][i].orient;
			
		k = l = 0;
		for(j=net[0][i].start; j<=net[0][i].end; j++) {			
			while(net[0][i].net1[k] == '-') {
				k++;
				if(net[0][i].net2[k] != '-')
					l++;
			}
			if(j >= start[ref-1] && j <= end[ref-1]) {
				if(net[0][i].net1[k] != 'X')
					seqs[ref-1][2][j - start[ref-1]] = net[0][i].net2[k];
				if(net[0][i].orient == '+') {
					if(outgroup_start[ref-1] == -1 || net[0][i].start2 + l < outgroup_start[ref-1])
						outgroup_start[ref-1] = net[0][i].start2 + l;
					if(outgroup_end[ref-1] == -1 || net[0][i].start2 + l > outgroup_end[ref-1])
						outgroup_end[ref-1] = net[0][i].start2 + l;
				}
				else {
					if(outgroup_start[ref-1] == -1 || net[0][i].start2 - l < outgroup_start[ref-1])
						outgroup_start[ref-1] = net[0][i].start2 - l;
					if(outgroup_end[ref-1] == -1 || net[0][i].start2 - l > outgroup_end[ref-1])
						outgroup_end[ref-1] = net[0][i].start2 - l;
				}
			}
			else if(j > end[ref-1]) {
				break;
			}
			k++;
			if(net[0][i].net2[k] != '-')
				l++;
		}
	}
	
	return 1;
}

void free_aln(aln_t *aln) {
	if(aln->next != NULL)
		free_aln(aln->next);
	
	free(aln);

	return;
}

int aln2_pos(chain_t c, int pos1) {
	aln_t *aln;
	int i, j, pos2;
	
	aln = c.aln;
	while(aln != NULL) {
		if(pos1 < aln->start1 + aln->len1) {
			j = aln->start1 - 1;
			if(c.orient == '+') {
				pos2 = aln->start2 - 1;
				for(i=0; j<pos1; i++) {
					if(aln->text1[i] != '-')
						j++;
					if(aln->text2[i] != '-')
						pos2++;
				}
			}
			else {
				pos2 = aln->src_len2 - aln->start2 + 1;
				for(i=0; j<pos1; i++) {
					if(aln->text1[i] != '-')
						j++;
					if(aln->text2[i] != '-')
						pos2--;
				}
			}
			return pos2;
		}
		else if(pos1 < aln->next->start1) {
			if(c.orient == '+')
				pos2 = aln->start2 + aln->len2 + (pos1 - (aln->start1 + aln->len1));
			else
				pos2 = aln->src_len2 - (aln->start2 + aln->len2) - (pos1 - (aln->start1 + aln->len1));
			return pos2;
		}
		
		aln = aln->next;
	}
	
	return -1;
}

void divided_index(chain_t c) {
	int len;
	
	len = c.end1 - c.start1 + 1;
	if(len <= 5000) {
		d_beg_index[0] = c.start2;
		d_end_index[0] = c.end2;
		d_index_num = 1;
	}
	else {
		d_index_num = 0;
		site_beg[0] = 0;
		site_end[0] = 4999;
		while(site_end[0] < len) {
			d_beg_index[d_index_num] = aln2_pos(c, site_beg[0] + c.start1 - 1);
			d_end_index[d_index_num] = aln2_pos(c, site_end[0] + c.start1 - 1);
			d_index_num++;
			if(site_end[0] == len - 1)
				break;
			else if(site_end[0] + 4000 >= len - 1) {
				site_beg[0] = len - 5000;
				site_end[0] = len - 1  ;
			}
			else {
				site_beg[0] = site_end[0] - 1000;
				site_end[0] = site_end[0] + 4000;
			}
		}
	}
}

float determine_directionality() {
	int md_m1, md_n1, md_m2, md_n2, i;
	float p, p1, p2, V, T;
	
	md_m1 = md_n1 = md_m2 = md_n2 = 0;
	for(i=beg_index+1; i<=end_index; i++) {
		if(IS_ref[i] == 0) {
			if(IS[2][i] == 1)
				md_m1++;
			else if(IS[2][i] == 2)
				md_n1++;
		}
		else if(IS_ref[i] == 1) {
			if(IS[2][i] == 1)
				md_m2++;
			else if(IS[2][i] == 2)
				md_n2++;
		}
	}

	p = (float)(md_n1 + md_n2) / (float)(md_m1 + md_m2 + md_n1 + md_n2);
	p1 = (float)md_n1 / (float)(md_m1 + md_n1);
	p2 = (float)md_n2 / (float)(md_m2 + md_n2);
	V = ((1.0/(float)(md_m1 + md_n1)) + (1.0/(float)(md_m2 + md_n2))) * p * (1.0 - p);
	if(V == 0.0 || (md_m1 + md_n1) == 0 || (md_m2 + md_n2) == 0)
		T = 0.0;
	else
		T = ((float)(p1 - p2)) / sqrtf(V);
	
	//printf("\n\nm1 = %d; n1 = %d; m2 = %d; n2 = %d; V = %f; T = %f\n\n", md_m1, md_n1, md_m2, md_n2, V, T);
	
	return T;
}

void print_block_index() {
	int i, j;

	for(i=0; i<2; i++) {
		if(orthologs_block_num[i] == 0)
			printf("\tNAN");
		else {
			printf("\t");
			for(j=0; j<orthologs_block_num[i]-1; j++) {
				printf("%d,", orthologs_block[i][j]);
			}
			printf("%d", orthologs_block[i][j]);
		}
	}
	printf("\n");
}

int read_chain(char *chain_fn) {
	FILE *fp;
	char *not_eof;
	chain_t c;
	int index;
	aln_t **aln_ptr, *aln;
	int i, match, mismatch, triplet_status, aln_len;
	float d_conf;

	fp = fopen(chain_fn, "r");
	while((not_eof = fgets(buf, 1000000, fp)) && buf[0] == '#')
		;
	while(not_eof) {
		sscanf(buf, "%d %s %d %d %s %d %d %c", &index, c.chr1, &c.start1, &c.end1, c.chr2, &c.start2, &c.end2, &c.orient);
		// test length
		if((c.end1 - c.start1 + 1) > 1000000 || (c.end2 - c.start2 + 1) > 1000000) {
			printf("chain is too long : %d - %d\n", c.end1 - c.start1 + 1, c.end2 - c.start2 + 1);
			exit(1);
		}
		fgets(buf, 1000000, fp);
		aln_ptr = &c.aln;
		while(buf[0] == 'a') {
			*aln_ptr = (aln_t *)malloc(sizeof(aln_t));
			fgets(buf, 1000000, fp);
			sscanf(buf, "s %s %d %d %c %d %s", c.chr1, &(*aln_ptr)->start1, &(*aln_ptr)->len1, &c.orient, &(*aln_ptr)->src_len1, (*aln_ptr)->text1);
			fgets(buf, 1000000, fp);
			sscanf(buf, "s %s %d %d %c %d %s", c.chr2, &(*aln_ptr)->start2, &(*aln_ptr)->len2, &c.orient, &(*aln_ptr)->src_len2, (*aln_ptr)->text2);
			(*aln_ptr)->next = NULL;
			aln_ptr = &((*aln_ptr)->next);
			fgets(buf, 1000000, fp);
			not_eof = fgets(buf, 1000000, fp);
		}
		
		start[0] = c.start1;
		end[0] = c.end1;
		sites_num[0] = c.end1 - c.start1 + 1;
		start[1] = c.start2;
		end[1] = c.end2;
		sites_num[1] = c.end2 - c.start2 + 1;
		
		//calculate identity
		aln = c.aln;
		match = mismatch = 0;
		while(aln != NULL) {
			aln_len = strlen(aln->text1);
			for(i=0; i<aln_len; i++) {
				if(toupper(aln->text1[i]) == toupper(aln->text2[i]))
					match++;
				else
					mismatch++;
			}
			aln = aln->next;
		}
		identity = (double)match / (double)(match + mismatch);
		
		divided_index(c);

		site_beg[0] = 0;
		site_beg[1] = 0;
		long_seqs = 0;
		outgroup_start[0] = outgroup_start[1] = outgroup_end[0] = outgroup_end[1] = -1;
		strcpy(outgroup_name[0], "NAN");
		strcpy(outgroup_name[1], "NAN");
		outgroup_orient[0] = outgroup_orient[1] = 'N';
		orthologs_block_num[0] = orthologs_block_num[1] = 0;
		if(process_chain(c, 1) == 1) {	 
			if(process_chain(c, 2) == 1) 
				triplet_status = 3;
			else 
				triplet_status = 1;
		}		
		else if(process_chain(c, 2) == 1) 		
			triplet_status = 2;
		else
			triplet_status = 0;
		
		// test conversion ratio
		site_end[0] = sites_num[0];
		if(triplet_status == 1 || triplet_status == 3) {
			find_IS(1);
			find_max_descent(1);
		}
		else {
			IS_num[0] = 0;
			m[0] = 0;
			n[0] = 0;
			max_descent[0] = 0;
		}
		site_end[1] = sites_num[1];
		if(triplet_status == 2 || triplet_status == 3) {
			find_IS(2);
			find_max_descent(2);
		}
		else {
			IS_num[1] = 0;
			m[1] = 0;
			n[1] = 0;
			max_descent[1] = 0;
		}
		d_conf = 0.0;
		if(triplet_status != 0) {
			combine_IS(c.orient);
			find_max_descent(3);
			if(triplet_status == 3)
				d_conf = determine_directionality();
		}
		if(triplet_status == 0 || (float)(abs(max_end1 - max_beg1) + 1) / (float)sites_num[0] > MAX_GC_RATIO) {
			printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%s\t%d\t%d\t%c\t%s\t%d\t%d\t%c", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], 0, 0, 0, 0, 0, 0, 0, 0, 0, identity, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, d_conf, outgroup_name[0], outgroup_start[0], outgroup_end[0], outgroup_orient[0], outgroup_name[1], outgroup_start[1], outgroup_end[1], outgroup_orient[1]);			// add block index
			print_block_index();
			for(i=5000; i<sites_num[0]; i+=4000) {
				printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%s\t%d\t%d\t%c\t%s\t%d\t%d\t%c", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], 0, 0, 0, 0, 0, 0, 0, 0, 0, identity, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, d_conf, outgroup_name[0], outgroup_start[0], outgroup_end[0], outgroup_orient[0], outgroup_name[1], outgroup_start[1], outgroup_end[1], outgroup_orient[1]);				print_block_index();
			}	
		}
		else if(sites_num[0] <= 5000) {
			printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%s\t%d\t%d\t%c\t%s\t%d\t%d\t%c", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], m[0], n[0], max_descent[0], m[1], n[1], max_descent[1], m[0] + m[1], n[0] + n[1], max_descent[2], identity, min_len, max_len, max_beg1, min_beg1, min_end1, max_end1, max_beg2, min_beg2, min_end2, max_end2, d_conf, outgroup_name[0], outgroup_start[0], outgroup_end[0], outgroup_orient[0], outgroup_name[1], outgroup_start[1], outgroup_end[1], outgroup_orient[1]);
			print_block_index();
		}
		else {
			long_seqs = 1;
			site_end[0] = 5000;
			i = 0;
			while(site_end[0] <= sites_num[0]) {
				if(triplet_status == 1 || triplet_status == 3) {
					find_IS(1);
					find_max_descent(1);
				}
				else {
					IS_num[0] = 0;
					m[0] = 0;
					n[0] = 0;
					max_descent[0] = 0;
				}
				if(triplet_status == 2 || triplet_status == 3) {
					if(c.orient == '+') {
						site_beg[1] = d_beg_index[i] - c.start2;
						site_end[1] = d_end_index[i] - c.start2;
					}
					else {
						site_beg[1] = d_end_index[i] - c.start2;
						site_end[1] = d_beg_index[i] - c.start2;
					}
					find_IS(2);
					find_max_descent(2);
				}
				else {
					IS_num[1] = 0;
					m[1] = 0;
					n[1] = 0;
					max_descent[1] = 0;
				}
				combine_IS(c.orient);
				find_max_descent(3);
				printf("%d\t%s\t%d\t%d\t%s\t%d\t%d\t%c\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%f\t%s\t%d\t%d\t%c\t%s\t%d\t%d\t%c", index, c.chr1, c.start1, c.end1, c.chr2, c.start2, c.end2, c.orient, sites_num[0], m[0], n[0], max_descent[0], m[1], n[1], max_descent[1], m[0] + m[1], n[0] + n[1], max_descent[2], identity, min_len, max_len, max_beg1, min_beg1, min_end1, max_end1, max_beg2, min_beg2, min_end2, max_end2, d_conf, outgroup_name[0], outgroup_start[0], outgroup_end[0], outgroup_orient[0], outgroup_name[1], outgroup_start[1], outgroup_end[1], outgroup_orient[1]);				print_block_index();
				
				if(site_end[0] == sites_num[0])
					break;
				else if(site_end[0] + 4000 >= sites_num[0]) {
					site_beg[0] = sites_num[0] - 5000;
					site_end[0] = sites_num[0];
				}
				else {
					site_beg[0] = site_end[0] - 1000;
					site_end[0] = site_end[0] + 4000;
				}
				i++;
			}
		}	
		
		//free aln
		free_aln(c.aln);
	}

	fclose(fp);
	
	return 0;
}

int main(int argc, char *argv[]) {
	if ( argc != 3 && argc != 4) {
		printf("md_quadruplet_orthologs chain-file orthologs_file <score-file> \n");
		return 1;
	}
	// read input file
	if(argc == 4) {
		read_score(argv[3]);
		if_rescoring = 1;
	}

	read_orthologs(argv[2]);
	read_chain(argv[1]);
	//printf("d_0 = %d, d_1 = %d, d_2 = %d, d_3 = %d\n", d_0, d_1, d_2, d_3);

	return 0;
}
